--[[
	---------------------------------------------------------
	#### Develop ####
    calca-Box v 2.0 - Lua application for JETI DC/DS transmitters   
    Copyright (c) 2024, Walter Loetscher
    All rights reserved
    License: Share alike
    Can be used and changed non commercial

	Changes: Complete Redesign
	----------------------------------------------------------
--]]
collectgarbage()
--------------------------------------------------------------------------------
-- Locals for application		
local trans11, lng, value
local batTypeA, batTypeB, cellCountA, cellCountB, alarmCap, alarmPrecap			-- LiPo = 1, LiIo = 2, LiFe = 3, BEC = 4, None = 5
local volt1, volt1Id, volt1Pa, volt2, volt2Id, volt2Pa
local volt = {rawA = 0, rawB = 0, corA = 0, corB = 0, compA = 0, compB = 0, graphA = 0, graphB = 0}
local amp = {A = 0, B = 0, comb, graph = 0, scale = 5, maxi = 0, init = 0}
local va, vb = {}, {}
local timer = {actual=0,second=0,init=0,sust=0,tempOff=0,errV1=0,errV2=0,initV1=0,initV2=0,main1=0,main2=0,main3=0}
local amp1, amp1Id, amp1Pa, amp2, amp2Id, amp2Pa
local ampState, volt1State, volt2State, change = 0, 0, 0, true
local scale = {botA = 0, topA = 0, botB = 0, topB = 0}
local inhib = {init = false, perc1 = false, perc2 = false, perc3 = false, sust = false}
local perc = {indexV1 = 0, indexV2 = 0, graphA = 50, graphB = 50, bat1 = 0, bat2 = 0, statA = "Off", statB = "Off"}
local alarm = {pre1=false,pre2=false,main1=false,main2=false,arm1=false,arm2=false,arm3=false}
local counter = {delay = 0, stateVolt = 0, initAmp = 0, peakAmp = 0, maxPeak = 3, wait = 0}
local red1,green1,blue1,red2,green2,blue2,red3,green3,blue3,red4,green4,blue4 = 0,0,0,0,0,0,0,0,0,0,0,0
local maxInitCurr, maxSustCurr, maxPeakCurr, initSum, batNameA, batNameB = 0,0,0,0,"RAS", "RAS"
local sensorLalist = { "..." }
local sensorIdlist = { "..." }
local sensorPalist = { "..." }
--------------------------------------------------------------------------------
-- Read and set translations
local function setLanguage()
    lng=system.getLocale()
    local file = io.readall("Apps/calca-Box/calca-Box.jsn")
    local obj = json.decode(file)
    if(obj) then
        trans11 = obj[lng] or obj[obj.default]
    end
end
--------------------------------------------------------------------------------
-- Telemetry and Data Verification
local function printTelem()															-- Main Telemetry Window
	lcd.setColor(230,230,230)														-- Background
	lcd.drawFilledRectangle(0,0,150,68,255)
	lcd.setColor(red3,green3,blue3)													-- Text Black
	lcd.drawRectangle(30,1,120,10)
	lcd.setColor(red4,green4,blue4)
	lcd.drawRectangle(30,21,120,10)
	lcd.setColor(0,0,0)	
	lcd.drawLine(0,32,45,32)
	lcd.drawLine(45,32,50,45)
	lcd.drawLine(50,45,120,45)
	lcd.drawLine(120,45,125,32)
	lcd.drawLine(125,32,150,32)	
	lcd.drawRectangle(55,48,95,10)
	lcd.drawLine (30,21,30,11)
	lcd.drawLine (149,21,149,11)
	lcd.drawLine (149,48,149,38)
	lcd.setColor(red1, green1, blue1)
	lcd.drawFilledRectangle(31,2,volt.graphA,8)
	lcd.setColor(red2, green2, blue2)
	lcd.drawFilledRectangle(31,22,volt.graphB,8)
	lcd.setColor(0,0,255)
	lcd.drawFilledRectangle(56,49,amp.graph,8)	
	lcd.setColor(red3,green3,blue3)
	lcd.drawText(2,0,string.format("%.2fV",volt.rawA),FONT_MINI)	
	lcd.setColor(red4,green4,blue4)
	lcd.drawText(2,20,string.format("%.2fV",volt.rawB),FONT_MINI)
	lcd.setColor(150,0,200)
	lcd.drawText(55,32,string.format("Alarm: %.3s%%",alarmCap),FONT_MINI)
	lcd.drawText(32,10,string.format("A-%.0fS:",cellCountA),FONT_MINI)
	lcd.drawText(92,10,string.format("B-%.0fS:",cellCountB),FONT_MINI)	
	lcd.setColor(0,0,0)
	lcd.drawText(0,10, trans11.text1, FONT_MINI)
	lcd.drawText(32,0, batNameA,FONT_MINI)
	lcd.drawText(32,20, batNameB,FONT_MINI)	
	if batTypeA > 3 then
	lcd.drawText(80,0,perc.statA,FONT_MINI)
	else
	lcd.drawText(80,0,string.format("%.3s%%",perc.graphA),FONT_MINI)
	end	
	if batTypeB > 3 then
	lcd.drawText(80,20,perc.statB,FONT_MINI)
	else
	lcd.drawText(80,20,string.format("%.3s%%",perc.graphB),FONT_MINI)
	end	
	lcd.drawText(0,34,trans11.current,FONT_MINI)
	lcd.drawText(5,42,string.format("%.2fA",amp.A + amp.B),FONT_BOLD)
	lcd.drawText(0,57,string.format("A: %.2fA",amp.A),FONT_MINI)
	lcd.drawText(45,57,string.format("B: %.2fA",amp.B),FONT_MINI)
	lcd.drawText(95,57,string.format("Max: %.1fA",amp.maxi),FONT_MINI)
	lcd.drawText(135,36,string.format("%.0fA", amp.scale),FONT_MINI)
	lcd.setColor(30,30,30)
	lcd.drawText(59,10,string.format("%.2fV",volt.corA),FONT_MINI)
	lcd.drawText(119,10,string.format("%.2fV",volt.corB),FONT_MINI)
end
--------------------------------------------------------------------------------
-- Read available sensors for user to select
local function readSensors()
    local sensors = system.getSensors()
    local format = string.format
    local insert = table.insert
    for i, sensor in ipairs(sensors) do
        if (sensor.label ~= "") then
            insert(sensorLalist, format("%s", sensor.label))
            insert(sensorIdlist, format("%s", sensor.id))
            insert(sensorPalist, format("%s", sensor.param))
        end
    end
end
----------------------------------------------------------------------
-- Actions when settings changed
local function vsChanged(value)
    local pSave = system.pSave
	voltSwitch = value
	pSave("voltSwitch", value)
end

local function caChanged(value)
    local pSave = system.pSave
	batTypeA = value
	pSave("batTypeA", value)
	change = true
end

local function cbChanged(value)
    local pSave = system.pSave
	batTypeB = value
	pSave("batTypeB", value)
	change = true
end

local function saChanged(value)
	cellCountA=value
	system.pSave("cellCountA",value)
end

local function sbChanged(value)
	cellCountB=value
	system.pSave("cellCountB",value)
end

local function acChanged(value)
	local pSave = system.pSave
	alarmCap=value
	pSave("alarmCap",value)
	change = true
end

local function v1Changed(value)
    local pSave = system.pSave
    local format = string.format
    volt1 = value
    volt1Id = format("%s", sensorIdlist[volt1])
    volt1Pa = format("%s", sensorPalist[volt1])
    if (volt1Id == "...") then
        volt1Id = 0
        volt1Pa = 0 
    end
    pSave("volt1", value)
    pSave("volt1Id", volt1Id)
    pSave("volt1Pa", volt1Pa)
end

local function v2Changed(value)
    local pSave = system.pSave
    local format = string.format
    volt2 = value
    volt2Id = format("%s", sensorIdlist[volt2])
    volt2Pa = format("%s", sensorPalist[volt2])
    if (volt2Id == "...") then
        volt2Id = 0
        volt2Pa = 0 
    end
    pSave("volt2", value)
    pSave("volt2Id", volt2Id)
    pSave("volt2Pa", volt2Pa)
end

local function a1Changed(value)
    local pSave = system.pSave
    local format = string.format
    amp1 = value
    amp1Id = format("%s", sensorIdlist[amp1])
    amp1Pa = format("%s", sensorPalist[amp1])
    if (amp1Id == "...") then
        amp1Id = 0
        amp1Pa = 0 
    end
    pSave("amp1", value)
    pSave("amp1Id", amp1Id)
    pSave("amp1Pa", amp1Pa)
end

local function a2Changed(value)
    local pSave = system.pSave
    local format = string.format
    amp2 = value
    amp2Id = format("%s", sensorIdlist[amp2])
    amp2Pa = format("%s", sensorPalist[amp2])
    if (amp2Id == "...") then
        amp2Id = 0
        amp2Pa = 0 
    end
    pSave("amp2", value)
    pSave("amp2Id", amp2Id)
    pSave("amp2Pa", amp2Pa)
end

local function initChanged(value)
	local pSave = system.pSave
	maxInitCurr=value
	pSave("maxInitCurr",value)
end
local function sustChanged(value)
	local pSave = system.pSave
	maxSustCurr=value
	pSave("maxSustCurr",value)
end
local function peakChanged(value)
	local pSave = system.pSave
	maxPeakCurr=value
	pSave("maxPeakCurr",value)
end
--------------------------------------------------------------------------------
local function calc_v()
	for i = 0, 100 do va[i] = 0 end
	if batTypeA == 1 then														-- LiPo Lithium Polymer
	va[0] = 0																	-- ********************
	va[22] = 1																	
	va[33] = 5																	-- einer Spannung wird die Prozentzahl zugeteilt
	va[42] = 10																	-- Volt Hundertstel zwischen 3,2 bis 4,2 Volt			
	va[46] = 15
	va[51] = 20
	va[53] = 25
	va[54] = 30
	va[55] = 35
	va[56] = 40
	va[57] = 45
	va[58] = 50
	va[60] = 55
	va[62] = 60
	va[66] = 65
	va[69] = 70
	va[72] = 75
	va[75] = 80
	va[80] = 85
	va[86] = 90
	va[90] = 95
	va[96] = 99
	va[100] = 100
	scale.botA = 3.2 scale.topA = 4.2
	elseif batTypeA == 2 then													-- LiIo Lithium Ionen
	va[0] = 0																	-- ******************
	va[40] = 5
	va[60] = 10																	-- Volt 2Hundertstel zwischen 2,2 bis 4,2 Volt
	va[69] = 15
	va[73] = 20																	
	va[75] = 25
	va[77] = 30
	va[78] = 50
	va[79] = 70
	va[81] = 85
	va[82] = 90
	va[90] = 95
	va[100] = 100
	scale.botA = 2.2 scale.topA = 4.2
	elseif batTypeA == 3 then													-- LiFe Lithium Eisen
	va[0] = 0																	-- ******************																	
	va[15] = 5
	va[30] = 15
	va[52] = 20																	-- Volt Fuenfzigstel zwischen 2,8 bis 3,3 Volt
	va[64] = 25
	va[70] = 30
	va[74] = 35
	va[78] = 40
	va[82] = 45
	va[84] = 50
	va[88] = 70
	va[90] = 80
	va[94] = 90
	va[98] = 95
	va[100] = 100
	scale.botA = 2.8 scale.topA = 3.30											-- Skala der Anzeige	
	else
	scale.botA = 4.5 scale.topA = 8.4
	end

	for i = 0, 100 do vb[i] = 0 end
	if batTypeB == 1 then														-- LiPo Lithium Polymer
	vb[0] = 0																	-- ********************
	vb[22] = 1																	
	vb[33] = 5																	-- einer Spannung wird die Prozentzahl zugeteilt
	vb[42] = 10																	-- Volt Hundertstel zwischen 3,2 bis 4,2 Volt			
	vb[46] = 15
	vb[51] = 20
	vb[53] = 25
	vb[54] = 30
	vb[55] = 35
	vb[56] = 40
	vb[57] = 45
	vb[58] = 50
	vb[60] = 55
	vb[62] = 60
	vb[66] = 65
	vb[69] = 70
	vb[72] = 75
	vb[75] = 80
	vb[80] = 85
	vb[86] = 90
	vb[90] = 95
	vb[96] = 99
	vb[100] = 100
	scale.botB = 3.2 scale.topB = 4.2
	elseif batTypeB == 2 then													-- LiIo Lithium Ionen
	vb[0] = 0																	-- ******************
	vb[40] = 5
	vb[60] = 10																	-- Volt 2Hundertstel zwischen 2,2 bis 4,2 Volt
	vb[69] = 15
	vb[73] = 20																	
	vb[75] = 25
	vb[77] = 30
	vb[78] = 50
	vb[79] = 70
	vb[81] = 85
	vb[82] = 90
	vb[90] = 95
	vb[100] = 100
	scale.botB = 2.2 scale.topB = 4.2
	elseif batTypeB == 3 then													-- LiFe Lithium Eisen
	vb[0] = 0																	-- ******************																	
	vb[15] = 5
	vb[30] = 15
	vb[52] = 20																	-- Volt Fuenfzigstel zwischen 2,8 bis 3,3 Volt
	vb[64] = 25
	vb[70] = 30
	vb[74] = 35
	vb[78] = 40
	vb[82] = 45
	vb[84] = 50
	vb[88] = 70
	vb[90] = 80
	vb[94] = 90
	vb[98] = 95
	vb[100] = 100
	scale.botB = 2.8 scale.topB = 3.30											-- Skala der Anzeige	
	else
	scale.botB = 4.5 scale.topB = 8.4
	end
	
	if batTypeA < 4 then
		local fillfrom, fillto, gap = 1,1,0
		while fillfrom < 100 do		
			while (va[fillfrom] > 0.0) and (fillfrom < 99) do
				fillfrom = fillfrom + 1
			end
			fillto = fillfrom
			while va[fillto+1] == 0.0 do
				fillto = fillto + 1
			end		
				gap = (va[fillto+1] - va[fillfrom-1]) / (fillto - fillfrom + 2)
					for i = fillfrom, fillto do
					va[i] = va[i-1] + gap
					end
			fillfrom = fillto + 2
		end
	end
	
	if batTypeB < 4 then
		local fillfrom, fillto, gap = 1,1,0
		while fillfrom < 100 do		
			while (vb[fillfrom] > 0.0) and (fillfrom < 99) do
				fillfrom = fillfrom + 1
			end
			fillto = fillfrom
			while vb[fillto+1] == 0.0 do
				fillto = fillto + 1
			end		
				gap = (vb[fillto+1] - vb[fillfrom-1]) / (fillto - fillfrom + 2)
					for i = fillfrom, fillto do
					vb[i] = vb[i-1] + gap
					end
			fillfrom = fillto + 2
		end
	end
	
end
--------------------------------------------------------------------------------
local function getVoltAmp()
	local voltA = system.getSensorByID(volt1Id, volt1Pa)
	if(voltA and voltA.valid) then
        volt.rawA = tonumber(string.format("%.2f", voltA.value))
	else volt.rawA = 0
	end
	local voltB = system.getSensorByID(volt2Id, volt2Pa)
	if(voltB and voltB.valid) then
        volt.rawB = tonumber(string.format("%.2f", voltB.value))
	else volt.rawB = 0
    end
	
	if batTypeA > 3 then
		volt.compA = volt.rawA
		else
		volt.compA = volt.rawA / cellCountA
	end
	
	if batTypeB > 3 then
		volt.compB = volt.rawB
		else
		volt.compB = volt.rawB / cellCountB
	end
		
	local ampA = system.getSensorByID(amp1Id, amp1Pa)
	if(ampA and ampA.valid) then
        amp.A = tonumber(string.format("%.2f", ampA.value))
	else amp.A = 0
    end
	local ampB = system.getSensorByID(amp2Id, amp2Pa)
	if(ampB and ampB.valid) then
        amp.B = tonumber(string.format("%.2f", ampB.value))
	else amp.B = 0
    end
	amp.comb = amp.A + amp.B
	if amp.comb > amp.scale then amp.scale = math.floor(amp.comb + 2) end
	if amp.comb > amp.maxi then amp.maxi = amp.comb end
end
--------------------------------------------------------------------------------
local function procVolt()
		volt.corA = volt.compA + (amp.A / 50)								-- Standard Korrektur für Einbruch
		volt.corB = volt.compB + (amp.B / 50)								-- Standard Korrektur für Einbruch
		
		perc.indexV1 = (volt.corA - scale.botA) / (scale.topA - scale.botA) * 100
		perc.indexV2 = (volt.corB - scale.botB) / (scale.topB - scale.botB) * 100
		if perc.indexV1 < 0 then perc.indexV1 = 0 end
		if perc.indexV2 < 0 then perc.indexV2 = 0 end
		if perc.indexV1 > 100 then perc.indexV1 = 100 end
		if perc.indexV2 > 100 then perc.indexV2 = 100 end
		
		if volt1State == 2 then
			
			if batTypeA == 4 then
				perc.bat1 = 100
				perc.statA = trans11.on
				if volt.rawA < 5 then alarm.pre1 = true alarm.main1 = false perc.bat1 = 65 perc.statA = trans11.low end
				if volt.rawA < 4.8 then alarm.pre1 = false alarm.main1 = true perc.bat1 = 25 perc.statA = trans11.off end
				if volt.rawA >= 5 then alarm.main1 = false alarm.pre1 = false end
				if volt.rawA > 5.2 then inhib.perc1 = false inhib.perc3 = false end
			elseif batTypeA == 5 then
				perc.bat1 = 0 alarm.main1 = true
			else			
				perc.bat1 = va[math.floor(perc.indexV1+0.5)]
				if perc.bat1 <= alarmPrecap then alarm.pre1 = true alarm.main1 = false end
				if perc.bat1 <= alarmCap then alarm.pre1 = false alarm.main1 = true end
				if perc.bat1 > alarmPrecap then alarm.main1 = false alarm.pre1 = false end
				if perc.bat1 > (alarmCap + 20) then inhib.perc1 = false inhib.perc3 = false end				
			end
				
		elseif volt1State < 2 then perc.bat1 = 0
		end
	
		if volt2State == 2 then
				
			if batTypeB == 4 then
				perc.bat2 = 100
				perc.statB = trans11.on
				if volt.rawB < 5 then alarm.pre2 = true alarm.main2 = false perc.bat2 = 25 perc.statB = trans11.low end
				if volt.rawB < 4.8 then alarm.pre2 = false alarm.main2 = true perc.bat2 = 0 perc.statB = trans11.off end
				if volt.rawB >= 5 then alarm.main2 = false alarm.pre2 = false end
				if volt.rawB > 5.2 then inhib.perc2 = false inhib.perc3 = false end
			elseif batTypeB == 5 then
				perc.bat2 = 0 alarm.main2 = true			
			else		
				perc.bat2 = vb[math.floor(perc.indexV2+0.5)]
				if perc.bat2 <= alarmPrecap then alarm.pre2 = true alarm.main2 = false end
				if perc.bat2 <= alarmCap then alarm.pre2 = false alarm.main2 = true end
				if perc.bat2 > alarmPrecap then alarm.main2 = false alarm.pre2 = false end
				if perc.bat2 > (alarmCap + 20) then inhib.perc2 = false inhib.perc3 = false end		
			end
		
		elseif volt2State < 2 then perc.bat2 = 0
		end
	
		if perc.bat1 >= (perc.graphA + 1) then perc.graphA = perc.graphA + 1 end
		if perc.bat1 <= (perc.graphA - 1) then perc.graphA = perc.graphA - 1 end
		if perc.bat2 >= (perc.graphB + 1) then perc.graphB = perc.graphB + 1 end
		if perc.bat2 <= (perc.graphB - 1) then perc.graphB = perc.graphB - 1 end
		
		volt.graphA = 118 * perc.bat1 / 100
		volt.graphB = 118 * perc.bat2 / 100		
end
--------------------------------------------------------------------------------
local function checkVoltState()													-- 0=Neustart, 1=Initializing, 2=Initialized, 3=Temp Off
	if volt1State == 0 then
		if volt.rawA > 0 then
			timer.initV1 = system.getTimeCounter()
			volt1State = 1
		end
	end
	if volt1State == 1 and system.getTimeCounter() >= (timer.initV1 + 1000) then
		volt1State = 2
	end
	if volt1State == 2 and volt.rawA == 0 then
		volt1State = 3
		timer.errV1 = system.getTimeCounter()
	end
	if volt1State == 3 then
		if volt.rawA > 0 then volt1State = 2
		else		
		if system.getTimeCounter() - timer.errV1 >= 10000 then volt1State = 0 end		
		end
	end
	
	if volt2State == 0 then
		if volt.rawB > 0 then
			timer.initV2 = system.getTimeCounter()
			volt2State = 1
		end
	end
	if volt2State == 1 and system.getTimeCounter() >= (timer.initV2 + 1000) then
		volt2State = 2
	end
	if volt2State == 2 and volt.rawB == 0 then
		volt2State = 3
		timer.errV2 = system.getTimeCounter()
	end
	if volt2State == 3 then
		if volt.rawB > 0 then volt2State = 2
		else		
		if system.getTimeCounter() - timer.errV2 >= 10000 then volt2State = 0 end		
		end
	end
end
--------------------------------------------------------------------------------
local function checkAmpState()														-- 0=Neustart, 1=Initializing, 2=Initialized, 3=Temp Off
	if ampState == 0 then
		if amp.comb > 0 then
			timer.init = system.getTimeCounter()
			ampState = 1
		end
	end
	if ampState == 1 and system.getTimeCounter() >= (timer.init + 1000) then
		ampState = 2
	end
	if ampState == 2 and amp.comb == 0 then
		ampState = 3
		timer.tempOff = system.getTimeCounter()
	end
	if ampState == 3 then
		if amp.comb > 0 then ampState = 2
		else		
		if system.getTimeCounter() - timer.tempOff >= 10000 then ampState = 0 end		
		end
	end
end
--------------------------------------------------------------------------------

local function voltAlarm()

	red1 = 0 green1 = 255 blue1 = 0
	red2 = 0 green2 = 255 blue2 = 0

	if alarm.pre1 == true then
		red1 = 255 green1 = 140 blue1 = 0										-- grün von 200 auf 140
	end 		
	if alarm.pre2 == true then
		red2 = 255 green2 = 140 blue2 = 0
	end
	
	if alarm.main1 == true then		
		red1 = 255 green1 = 0 blue1 = 0
		red3 = 255 green3 = 0 blue3 = 0		

		if alarm.arm1 == false then
			timer.main1 = system.getTime()			
			alarm.arm1 = true
		else
			if (system.getTime() > (timer.main1 + 10)) and (inhib.perc1 == false) then	-- Alarm Verzögerung 10 Sekunden
			print("Warning V1")
				if lng == "de" then
					system.playFile("/Apps/calca-Box/Box1LowVolt_D.wav", AUDIO_IMMEDIATE)
				else 
					system.playFile("/Apps/calca-Box/Box1LowVolt_E.wav", AUDIO_IMMEDIATE)
				end
			inhib.perc1 = true
			end
		end		
	else alarm.arm1 = false
	end
	
	if alarm.main2 == true then		
		red2 = 255 green2 = 0 blue2 = 0
		red4 = 255 green4 = 0 blue4 = 0
		
		if alarm.arm2 == false then
			timer.main2 = system.getTime()			
			alarm.arm2 = true
		else
			if (system.getTime() > (timer.main2 + 10)) and (inhib.perc2 == false) then	-- Alarm Verzögerung 10 Sekunden
			print("Warning V2")
				if lng == "de" then
					system.playFile("/Apps/calca-Box/Box2LowVolt_D.wav", AUDIO_IMMEDIATE)
				else 
					system.playFile("/Apps/calca-Box/Box2LowVolt_E.wav", AUDIO_IMMEDIATE)
				end
			inhib.perc2 = true
			end
		end		
	else alarm.arm2 = false
	end

	if (alarm.main1 == true) and (alarm.main2 == true) then
		if alarm.arm3 == false then
			timer.main3 = system.getTime()			
			alarm.arm3 = true
		else
			if (system.getTime() > (timer.main3 + 10)) and (inhib.perc3 == false) then	-- Alarm Verzögerung 10 Sekunden
			print("Warning V1 + V2")
				if lng == "de" then
					system.playFile("/Apps/calca-Box/BoxLowVolt_D.wav", AUDIO_IMMEDIATE)
				else 
					system.playFile("/Apps/calca-Box/BoxLowVolt_E.wav", AUDIO_IMMEDIATE)
				end
			inhib.perc3 = true
			end
		end		
	else alarm.arm3 = false
	end

end
--------------------------------------------------------------------------------
local function ampAlarm()
	amp.graph = 95 / amp.scale * amp.comb
	
	if inhib.init == false then													-- Alarm max Init (blocked Servos)
		if amp.comb > 0 then
			counter.initAmp = counter.initAmp + 1
		else counter.initAmp = 0 initSum = 0
		end
		if counter.initAmp > 9 then
			initSum = initSum + amp.comb
			if counter.initAmp == 19 then
				amp.init = initSum / 10
				if amp.init >= (maxInitCurr / 10) then
					print("Init Alarm")					
					if lng == "de" then
						system.playFile("/Apps/calca-Box/highCurr_D.wav", AUDIO_IMMEDIATE)
					else
						system.playFile("/Apps/calca-Box/highCurr_E.wav", AUDIO_IMMEDIATE)
					end
				end 
				inhib.init = true
				counter.initAmp = 0
				initSum = 0		
			end
		end
	end

	if ampState == 2 then															-- Alarm max Sustained
	
		if counter.wait > 200 then
		
			if amp.comb >= (maxPeakCurr / 10) then
				counter.peakAmp = counter.peakAmp + 1
				if counter.peakAmp >= counter.maxPeak then
					print("Peak Alarm")
					system.playFile("/Apps/calca-Box/A_peak.wav", AUDIO_IMMEDIATE)
					counter.peakAmp = -50
				end
			else counter.peakAmp = 0
			end
			
			if amp.comb >= (maxSustCurr/10) then

				if inhib.sust == false then
					inhib.sust = true
					timer.sust = system.getTimeCounter()
				end
				if system.getTimeCounter() > (timer.sust + 1000) then
					print("High Sustained Current")
					system.playFile("/Apps/calca-Box/A_sust.wav", AUDIO_IMMEDIATE)
					inhib.sust = false
				end
				
			end
		else counter.wait = counter.wait + 1
		end
	end

end
--------------------------------------------------------------------------------
local function everySecond()
	timer.second = timer.actual

	if volt1State + volt2State + ampState == 0 then
		counter.stateVolt = counter.stateVolt + 1
		if counter.stateVolt == 20 then
			inhib.init = false
		end
	else counter.stateVolt = 0
	end
	
	if (volt1State == 2) and (volt2State == 2) then
		if counter.delay < 25 then counter.delay = counter.delay + 1 end	
	end
	
	if perc.bat1 > alarmCap then
		red3 = 0 green3 = 0 blue3 = 0
	end	

	if perc.bat2 > alarmCap then
		red4 = 0 green4 = 0 blue4 = 0
	end
end
--------------------------------------------------------------------------------
local function loop()
	timer.actual = system.getTime()

	if change == true then
		calc_v()
		alarmPrecap = alarmCap + 5
		change = false 
		
		if batTypeA == 1 then batNameA = "LiPo"										-- Für Display
		elseif batTypeA == 2 then batNameA = "LiIo"
		elseif batTypeA == 3 then batNameA = "LiFe"
		elseif batTypeA == 4 then batNameA = "BEC"
		else batNameA = "None"
		end
		
		if batTypeB == 1 then batNameB = "LiPo"										-- Für Display
		elseif batTypeB == 2 then batNameB = "LiIo"
		elseif batTypeB == 3 then batNameB = "LiFe"
		elseif batTypeB == 4 then batNameB = "BEC"
		else batNameB = "None"
		end
		
	end

	getVoltAmp()
	procVolt()	
	checkAmpState()
	checkVoltState()
	voltAlarm()
	ampAlarm()
	
	if timer.actual >= timer.second + 1 then everySecond() end
    collectgarbage()
end
--------------------------------------------------------------------------------
-- Draw the main form (Application inteface)
local function initForm()  
    local form, addRow, addLabel = form, form.addRow ,form.addLabel
    local addIntbox, addCheckbox = form.addIntbox, form.addCheckbox
    local addSelectbox, addInputbox = form.addSelectbox, form.addInputbox
	local chemistry = {trans11.lipo, trans11.liio, trans11.life, trans11.bec, trans11.none}
	form.setTitle(trans11.appName.." "..trans11.version.."      W.L.")
	
	addRow(1)
    addLabel({label= trans11.entrys,font=FONT_NORMAL})
	
	addRow(2)
	addLabel({label= trans11.batChemA, font=FONT_BOLD})
	addSelectbox(chemistry, batTypeA, true, caChanged, {font=FONT_BOLD})
	
	addRow(2)
	addLabel({label= trans11.batChemB, font=FONT_BOLD})
	addSelectbox(chemistry, batTypeB, true, cbChanged, {font=FONT_BOLD})
	
	addRow(2)
	addLabel({label=trans11.cellCountA, font=FONT_BOLD})
	addIntbox(cellCountA,1,14,2,0,1,saChanged, {font=FONT_BOLD})
	
	addRow(2)
	addLabel({label=trans11.cellCountB, font=FONT_BOLD})
	addIntbox(cellCountB,1,14,2,0,1,sbChanged, {font=FONT_BOLD})
	
	addRow(2)
	addLabel({label=trans11.alarmCap, font=FONT_BOLD})
	addIntbox(alarmCap,1,99,20,0,1,acChanged, {font=FONT_BOLD})	

	addRow(1)
    addLabel({label= trans11.sensors,font=FONT_NORMAL})
	
	addRow(2)
    addLabel({label=trans11.volt1, width=180, font=FONT_BOLD})
    addSelectbox(sensorLalist, volt1, true, v1Changed, {font=FONT_BOLD})
	
	addRow(2)
    addLabel({label=trans11.volt2, width=180, font=FONT_BOLD})
    addSelectbox(sensorLalist, volt2, true, v2Changed, {font=FONT_BOLD})
	
	addRow(2)
    addLabel({label=trans11.amp1, width=180, font=FONT_BOLD})
    addSelectbox(sensorLalist, amp1, true, a1Changed, {font=FONT_BOLD})
	
	addRow(2)
    addLabel({label=trans11.amp2, width=180, font=FONT_BOLD})
    addSelectbox(sensorLalist, amp2, true, a2Changed, {font=FONT_BOLD})
	
	addRow(1)
	addLabel({label=trans11.ampLimit, font=FONT_NORMAL})
	addRow(2)
	addLabel({label=trans11.MaxInitCurrent, font=FONT_BOLD})                  -- Current in one tenth Ampere!
	addIntbox(maxInitCurr,1,100,10,1,1,initChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label=trans11.MaxSustCurrent, font=FONT_BOLD})
	addIntbox(maxSustCurr,1,100,15,1,1,sustChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label=trans11.MaxPeakCurrent, font=FONT_BOLD})
	addIntbox(maxPeakCurr,1,500,40,1,1,peakChanged, {font=FONT_BOLD})
        
    addRow(1)
    addLabel({label="Powered by Swiss-Aerodesign.com - "..trans11.version.." ", font=FONT_MINI, alignRight=true})
end
--------------------------------------------------------------------------------
local function init()
    local pLoad = system.pLoad

	batTypeA = pLoad("batTypeA", 1)
	batTypeB = pLoad("batTypeB", 1)
	
	cellCountA = pLoad("cellCountA",2)										-- Number of Cells
	cellCountB = pLoad("cellCountB",2)
	
	alarmCap = pLoad("alarmCap", 20)
	
	volt1 = pLoad("volt1", 0)
	volt1Id = pLoad("volt1Id", 0)
	volt1Pa = pLoad("volt1Pa", 0)
	
	volt2 = pLoad("volt2", 0)
	volt2Id = pLoad("volt2Id", 0)
	volt2Pa = pLoad("volt2Pa", 0)
	
	amp1 = pLoad("amp1", 0)
	amp1Id = pLoad("amp1Id", 0)
	amp1Pa = pLoad("amp1Pa", 0)
	
	amp2 = pLoad("amp2", 0)
	amp2Id = pLoad("amp2Id", 0)
	amp2Pa = pLoad("amp2Pa", 0)
	
	maxInitCurr = pLoad("maxInitCurr", 10)
	maxSustCurr = pLoad("maxSustCurr", 15)
	maxPeakCurr = pLoad("maxPeakCurr", 40)
		
	system.registerForm(1, MENU_MAIN, trans11.appName, initForm)
	system.registerTelemetry(1,trans11.appName .. trans11.version,2,printTelem)
    readSensors()

    collectgarbage()
end
--------------------------------------------------------------------------------
setLanguage()
collectgarbage()
return {init=init, loop=loop, author="Walter Loetscher", version=trans11.version, name=trans11.appName}
